@@ -361,7 +361,7 @@ local function construct_stanza_id(room, stanza)
 end
 local function deconstruct_stanza_id(room, stanza)
 	local from_jid_possiblybare, to_nick = stanza.attr.from, stanza.attr.to;
-	local from_jid, id, to_jid_hash = (base64.decode(stanza.attr.id) or ""):match("^(.+)%z(.*)%z(.+)$");
+	local from_jid, id, to_jid_hash = (base64.decode(stanza.attr.id) or ""):match("^(%Z+)%z(%Z*)%z(.+)$");
 	local from_nick = room._jid_nick[from_jid];
 
 	if not(from_nick) then return; end
@@ -668,6 +668,14 @@ function room_mt:process_form(origin, stanza)
 	if form.attr.type == "cancel" then origin.send(st.reply(stanza)); return; end
 	if form.attr.type ~= "submit" then origin.send(st.error_reply(stanza, "cancel", "bad-request", "Not a submitted form")); return; end
 
+	if form.tags[1] == nil then
+		-- instant room
+		if self.save then self:save(true); end
+		origin.send(st.reply(stanza));
+		return true;
+	end
+
+
 	local fields = self:get_form_layout():data(form);
 	if fields.FORM_TYPE ~= "http://jabber.org/protocol/muc#roomconfig" then origin.send(st.error_reply(stanza, "cancel", "bad-request", "Form is not of type room configuration")); return; end
 
@@ -736,7 +744,7 @@ function room_mt:process_form(origin, stanza)
 
 	if dirty or whois_changed then
 		local msg = st.message({type='groupchat', from=self.jid})
-			:tag('x', {xmlns='http://jabber.org/protocol/muc#user'}):up()
+			:tag('x', {xmlns='http://jabber.org/protocol/muc#user'});
 
 		if dirty then
 			msg.tags[1]:tag('status', {code = '104'}):up();
@@ -745,6 +753,7 @@ function room_mt:process_form(origin, stanza)
 			local code = (whois == 'moderators') and "173" or "172";
 			msg.tags[1]:tag('status', {code = code}):up();
 		end
+		msg:up();
 
 		self:broadcast_message(msg, false)
 	end
