@@ -49,6 +49,14 @@ package.preload["util.ztact"] = function ()
 end;
 
 function check_dependencies()
+	if _VERSION ~= "Lua 5.1" then
+		print "***********************************"
+		print("Unsupported Lua version: ".._VERSION);
+		print("Only Lua 5.1 is supported.");
+		print "***********************************"
+		return false;
+	end
+
 	local fatal;
 	
 	local lxp = softreq "lxp"
@@ -91,6 +99,10 @@ function check_dependencies()
 				["luarocks"] = "luarocks install luasec";
 				["Source"] = "http://www.inf.puc-rio.br/~brunoos/luasec/";
 			}, "SSL/TLS support will not be available");
+	elseif not _G.ssl then
+		_G.ssl = ssl;
+		_G.ssl.context = require "ssl.context";
+		_G.ssl.x509 = softreq "ssl.x509";
 	end
 	
 	local encodings, err = softreq "util.encodings"
